package cs451.net.event;

import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.Objects;

public class Message implements Comparable {

    public enum TYPE {
        NONE("NONE", 'N'),
        ERR("ERR", 'E'),
        DATA("DATA", ' '),
        ACK("ACK", 'A'),
        BCST("BCST", 'B');

        public final Character c;
        public final String tag;
        TYPE(String tag, Character c){
            this.tag = tag;
            this.c = c;
        }

    }

    public int src = -1;
    public final int id;
    public TYPE tpe;

    public static Message DMSG(Integer id){
        return new Message(id,TYPE.DATA);
    }
    public static Message DMSG(Integer id, Integer src) {
        return new Message(id,TYPE.DATA,src);
    }

    public static Message TMSG(Integer mess_id, TYPE tpe) {
        return new Message(mess_id, tpe);
    }

    public static Message EMPTY(){
        return new Message(-1, TYPE.NONE);
    }

    private static TYPE CharacterToTpe(Character c){
        return Arrays.stream(TYPE.values()).filter(type -> type.c==c).findFirst().orElse(TYPE.NONE);
    }
    public static Message FromBuffer(ByteBuffer b) {
        Character tpe = b.getChar();
        Integer id = b.getInt();
        Integer src = b.getInt();
        return new Message(id,CharacterToTpe(tpe),src);
    }

    public void ToBuffer(ByteBuffer b){
        b.putChar(tpe.c);
        b.putInt(id);
        b.putInt(src);
    }

    public Message(Message m) {
        this.id = m.id;
        this.tpe = m.tpe;
        this.src = m.src;
    }
    private Message(Integer id, TYPE tpe){
        this.id = id;
        this.tpe = tpe;
    }
    private Message(Integer id, TYPE tpe, Integer src){
        this.id = id;
        this.tpe = tpe;
        this.src = src;
    }

    @Override
    public String toString() {
        switch(tpe) {
            case DATA:
                return src+"-"+ id;
            default:
                return src+"-"+tpe.c + id;
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Message m = (Message) obj;
        return id == m.id && src == m.src;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id,src);
    }

    @Override
    public int compareTo(Object obj) {
        final Message m = (Message) obj;
        return id-m.id;
    }
}
