package cs451.net.handler;

import cs451.net.NetManager;
import cs451.net.event.NetEvent;
import cs451.net.event.NetEventType;
import cs451.parser.Host;
import cs451.parser.Parser;
import cs451.tools.Logger;

public abstract class NetEventHandler {

    /*
     * 1 - 50ms    - 50ms
     * 2 - 400ms   - 450ms
     * 3 - 1350ms  - 1800ms
     * 4 - 3200ms  - 5000ms
     */
    public static final int MS_WAIT = 50;
    public static final int MAX_TRIES = 5;

    public static final Integer SEND_WINDOW = 3;



    private final Class<? extends NetEventHandler> deliverLayer;
    private final Class<? extends NetEventHandler> broadcastLayer;


    NetEventHandler(Class<? extends NetEventHandler> deliverLayer, Class<? extends NetEventHandler> broadcastLayer) {
        this.deliverLayer = deliverLayer;
        this.broadcastLayer = broadcastLayer;
    }

    public void deliverNextSync(NetEvent ne){
        NetManager.deliverSync(deliverLayer,ne);
    }
    public void sendNextSync(NetEvent ne){
        NetManager.sendSync(broadcastLayer,ne);
    }

    public void deliverNext(NetEvent ne){
        NetManager.deliver(deliverLayer,ne);
    }
    public void sendNext(NetEvent ne){
        NetManager.send(broadcastLayer,ne);
    }

    public void onEvent(NetEventType et, NetEvent ne){
        switch (et){
            case DLVR: deliver(ne);
                break;
            case SEND: send(ne);
                break;
            case CRSH: crash(ne);
                break;
            case RCVR: recover(ne);
                break;
            default:
                Logger.error("Unhandled EventType:"+et);
        }
    }

    public void send(NetEvent ne){
        NetManager.error(NetEventType.SEND,new Exception("Default Handler"));
    }

    public void deliver(NetEvent ne){
        NetManager.error(NetEventType.DLVR,new Exception("Default Handler"));
    }

    public void crash(NetEvent ne){
        NetManager.error(NetEventType.CRSH,new Exception("Default Handler"));
    }

    public void recover(NetEvent ne){
        NetManager.error(NetEventType.RCVR,new Exception("Default Handler"));
    }

    public void start(Host h, Parser p) {}

    public void stop() {}

    public boolean isDone(){ return true;}
}
