package cs451.net.handler;

import cs451.net.NetManager;
import cs451.net.event.Message;
import cs451.net.event.NetEvent;
import cs451.net.event.NetEventType;
import cs451.parser.Host;
import cs451.parser.Parser;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.PriorityBlockingQueue;
import java.util.concurrent.atomic.AtomicInteger;

public class NetHandlerFIFO extends NetEventHandler {

    private final AtomicInteger sn = new AtomicInteger(1);
    private final Map<Integer,Integer> rsn = new ConcurrentHashMap<>();
    private final PriorityBlockingQueue<Message> pending = new PriorityBlockingQueue<>();

    private Host me;

    public NetHandlerFIFO(Class<? extends NetEventHandler> deliverLayer, Class<? extends NetEventHandler> broadcastLayer) {
        super(deliverLayer,broadcastLayer);
    }

    @Override
    public void send(NetEvent ne) {
        Integer snv = sn.getAndIncrement();
        NetManager.complete(NetEventType.SEND, NetEvent.Message(me, snv, Message.TYPE.BCST));
        sendNext(NetEvent.Message(me.getId(),snv));
    }

    private void deliverIf(){
        synchronized (pending) {
            pending.removeIf(hmp -> {
                Integer crsn = rsn.getOrDefault(hmp.src, 1);
                if (hmp.id == crsn) {
                    deliverNextSync(NetEvent.Message(hmp));
                    NetManager.complete(NetEventType.DLVR,NetEvent.Message(hmp));
                    rsn.put(hmp.src, crsn + 1);
                    return true;
                }
                return false;
            });
        }
    }

    @Override
    public void deliver(NetEvent ne) {
        pending.add(ne.message);
        deliverIf();
    }

    @Override
    public void start(Host h, Parser p) {
        me = h;
    }
}
