package cs451.net.handler;

import cs451.net.event.Message;
import cs451.net.NetManager;
import cs451.net.event.*;
import cs451.parser.Host;
import cs451.parser.Parser;

import java.io.IOException;
import java.net.*;
import java.nio.ByteBuffer;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicBoolean;

public class NetHandlerSCKT extends NetEventHandler {

    private static final Integer BUFF_SIZE = 128;

    private List<Host> hosts;

    private DatagramSocket socket = null;
    private final AtomicBoolean receiving = new AtomicBoolean(true);


    public NetHandlerSCKT(Class<? extends NetEventHandler> deliverLayer, Class<? extends NetEventHandler> broadcastLayer) {
        super(deliverLayer,broadcastLayer);
    }

    @Override
    public void send(NetEvent event) {
        if(socket.isClosed()){
            NetManager.error(NetEventType.SEND,new SocketException("Socket Closed"));
            return;
        }
        ByteBuffer b = ByteBuffer.allocate(BUFF_SIZE);
        event.message.ToBuffer(b);
        DatagramPacket datagram = new DatagramPacket(b.array(), b.position(), event.peer.getAddr());
        try {
            socket.send(datagram);
        } catch (IOException e) {
            NetManager.error(NetEventType.SEND,e);
        }
    }

    @Override
    public void deliver(NetEvent event) {
        if(socket.isClosed()){
            NetManager.error(NetEventType.DLVR,new SocketException("Socket Closed"));
            return;
        }

        byte[] buff = new byte[BUFF_SIZE];
        DatagramPacket datagram = new DatagramPacket(buff, buff.length);
        try {
            socket.receive(datagram);
            Optional<Integer> rhid = hosts.stream().filter(h -> h.getAddr().equals(datagram.getSocketAddress())).map(Host::getId).findFirst();

            deliverNext(NetEvent.Message(
                            new Host((InetSocketAddress) datagram.getSocketAddress(), rhid.orElse(-1)),
                            Message.FromBuffer(ByteBuffer.wrap(datagram.getData()))
            ));
        } catch (IOException e) {
            NetManager.error(NetEventType.DLVR,e);
        }
    }


    @Override
    public void start(Host h, Parser p) {
        Objects.requireNonNull(h);
        Objects.requireNonNull(p);
        hosts = p.hosts();

        try {
            this.socket = new DatagramSocket(h.getPort());
        } catch (SocketException e) {
            throw new Error(e);
        }
        receiving.set(true);
        new Thread(() -> {
            while (receiving.get()) {
                this.deliver(NetEvent.EMPTY());
            }
        }).start();
    }

    @Override
    public void stop() {
        socket.close();
        receiving.set(false);
    }
}
