package cs451.net.handler;

import cs451.net.event.Message;
import cs451.net.NetManager;
import cs451.net.event.*;
import cs451.tools.Pair;

import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;


public class NetHandlerSL extends NetEventHandler {

    private static final Set<Message> EMPTY_SET = ConcurrentHashMap.newKeySet();

    Set<Integer> hasTimeout = ConcurrentHashMap.newKeySet();
    Map<Integer, Set<Message>> hasAck = new ConcurrentHashMap<>();
    Map<Pair<Integer, Message>, Integer> retryCounter = new ConcurrentHashMap<>();

    public NetHandlerSL(Class<? extends NetEventHandler> deliverLayer, Class<? extends NetEventHandler> broadcastLayer) {
        super(deliverLayer,broadcastLayer);
    }


    private void handleTimeout(NetEvent ne) {
        if (hasAck.getOrDefault(ne.peer.getId(),EMPTY_SET).contains(ne.message)) {
            return;
        }
        if (retryCounter.compute(new Pair<>(ne.peer.getId(),ne.message), (k, ov) -> (ov == null) ? 2 : ov + 1) >= MAX_TRIES) {
            timeout(ne);
        } else {
            send(ne);
        }
    }

    private void checkTimeout(NetEvent ne) {
        new Thread(()->{
            Integer t = retryCounter.getOrDefault(new Pair<>(ne.peer.getId(), ne.message), 1);
            new Timer().schedule(
                new TimerTask() {
                    @Override
                    public void run () {
                        handleTimeout(ne);
                    }
                }, MS_WAIT*t*t*t);
        }).start();
    }

    @Override
    public void send(NetEvent ne) {
        if (hasTimeout.contains(ne.peer.getId())) {
            return;
        }
        sendNext(ne);
        checkTimeout(ne);
    }

    @Override
    public void deliver(NetEvent ne) {
        if (hasTimeout.contains(ne.peer.getId())) {
            return;
        }
        switch (ne.message.tpe){
            case ACK:
                hasAck.compute(ne.peer.getId(),(k, s) -> {
                    if(s == null) s = ConcurrentHashMap.newKeySet();
                    s.add(ne.message);
                    return s;
                });
                break;
            case DATA:
                sendNext(NetEvent.MessageACK(ne.peer,ne.message));
                deliverNext(ne);
                break;
            default:
                break;
        }
    }

    private void timeout(NetEvent event) {
        if(hasTimeout.add(event.peer.getId())) {
            NetManager.crash(NetHandlerURB.class, event);
        }
    }

}
