package cs451.net.handler;

import cs451.net.event.NetEvent;
import cs451.parser.Host;
import cs451.parser.Parser;

import java.util.concurrent.atomic.AtomicInteger;

public class NetHandlerTOPL extends NetEventHandler {

    private final AtomicInteger toSend = new AtomicInteger(0);
    private final AtomicInteger delivered = new AtomicInteger(0);
    private final AtomicInteger waiting = new AtomicInteger(0);

    private Host me;

    public NetHandlerTOPL(Class<? extends NetEventHandler> deliverLayer, Class<? extends NetEventHandler> broadcastLayer) {
        super(deliverLayer, broadcastLayer);
    }


    private void sendIf(){
        synchronized (waiting) {
            while (waiting.get() < SEND_WINDOW && toSend.get() > 0) {
                toSend.getAndDecrement();
                waiting.getAndIncrement();
                sendNextSync(NetEvent.EMPTY());
            }
        }
    }

    @Override
    public void send(NetEvent ne) {
        sendIf();
    }

    @Override
    public void deliver(NetEvent ne) {
        if (ne.message.src == me.getId()) {
            synchronized (waiting) {
                delivered.getAndIncrement();
                waiting.getAndDecrement();
            }
            sendIf();
        }
    }

    @Override
    public boolean isDone() {
        return toSend.get()==0 && waiting.get()==0;
    }

    @Override
    public void start(Host h, Parser p) {
        me = h;
        toSend.set(p.messageCount());
    }
}
