package cs451.net.handler;

import cs451.net.event.NetEvent;
import cs451.parser.Host;
import cs451.parser.Parser;
import cs451.tools.Pair;

import java.util.Map;
import java.util.Set;
import java.util.concurrent.*;


public class NetHandlerURB extends NetEventHandler {


    private final Set<Integer> correct = ConcurrentHashMap.newKeySet();
    private final Map<Pair<Integer,Integer>, Set<Integer>> ack = new ConcurrentHashMap<>();
    private final Set<Pair<Integer,Integer>> delivered = ConcurrentHashMap.newKeySet();
    private final Set<Pair<Integer,Integer>> pending = ConcurrentHashMap.newKeySet();

    private Integer myId;


    public NetHandlerURB(Class<? extends NetEventHandler> deliverLayer, Class<? extends NetEventHandler> broadcastLayer) {
        super(deliverLayer,broadcastLayer);
    }

    private void deliverIf(){
        pending.stream().filter(smp -> ack.getOrDefault(smp,ConcurrentHashMap.newKeySet()).containsAll(correct)).
                filter(delivered::add).
                forEach(smp -> deliverNext(NetEvent.Message(smp.first(),smp.second())));
    }

    @Override
    public void send(NetEvent ne) {
        pending.add(new Pair<>(myId, ne.message.id));
        ne.message.src = myId;
        sendNext(ne);
    }

    @Override
    public void deliver(NetEvent ne) {
        ack.compute(new Pair<>(ne.message.src,ne.message.id),(k, v) -> {
            if(v == null){
                v = ConcurrentHashMap.newKeySet();
            }
            v.add(ne.peer.getId());
            return v;
        });
        if(pending.add(new Pair<>(ne.message.src,ne.message.id))){
            sendNext(ne);
        }
        deliverIf();
    }

    public void crash(NetEvent ne) {
        correct.remove(ne.peer.getId());
        deliverIf();
    }

    public void start(Host h, Parser p) {
        p.hosts().forEach(ch-> correct.add(ch.getId()));
        myId = h.getId();
    }
}